@@ -10,13 +10,15 @@ use crate::{
 };
 use anyhow::Result;
 use asyncgit::{
-    sync::cred::{
-        extract_username_password, need_username_password,
-        BasicAuthCredential,
+    sync::{
+        cred::{
+            extract_username_password, need_username_password,
+            BasicAuthCredential,
+        },
+        get_default_remote,
     },
-    sync::DEFAULT_REMOTE_NAME,
-    AsyncNotification, AsyncPush, PushProgress, PushProgressState,
-    PushRequest,
+    AsyncNotification, AsyncPush, PushRequest, RemoteProgress,
+    RemoteProgressState, CWD,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
@@ -31,8 +33,9 @@ use tui::{
 ///
 pub struct PushComponent {
     visible: bool,
+    force: bool,
     git_push: AsyncPush,
-    progress: Option<PushProgress>,
+    progress: Option<RemoteProgress>,
     pending: bool,
     branch: String,
     queue: Queue,
@@ -51,6 +54,7 @@ impl PushComponent {
     ) -> Self {
         Self {
             queue: queue.clone(),
+            force: false,
             pending: false,
             visible: false,
             branch: String::new(),
@@ -66,35 +70,41 @@ impl PushComponent {
     }
 
     ///
-    pub fn push(&mut self, branch: String) -> Result<()> {
+    pub fn push(
+        &mut self,
+        branch: String,
+        force: bool,
+    ) -> Result<()> {
         self.branch = branch;
+        self.force = force;
         self.show()?;
-        if need_username_password(DEFAULT_REMOTE_NAME)? {
-            let cred = extract_username_password(DEFAULT_REMOTE_NAME)
-                .unwrap_or_else(|_| {
+        if need_username_password()? {
+            let cred =
+                extract_username_password().unwrap_or_else(|_| {
                     BasicAuthCredential::new(None, None)
                 });
             if cred.is_complete() {
-                self.push_to_remote(Some(cred))
+                self.push_to_remote(Some(cred), force)
             } else {
                 self.input_cred.set_cred(cred);
                 self.input_cred.show()
             }
         } else {
-            self.push_to_remote(None)
+            self.push_to_remote(None, force)
         }
     }
 
     fn push_to_remote(
         &mut self,
         cred: Option<BasicAuthCredential>,
+        force: bool,
     ) -> Result<()> {
         self.pending = true;
         self.progress = None;
         self.git_push.request(PushRequest {
-            //TODO: find tracking branch name
-            remote: String::from(DEFAULT_REMOTE_NAME),
+            remote: get_default_remote(CWD)?,
             branch: self.branch.clone(),
+            force,
             basic_credential: cred,
         })?;
         Ok(())
@@ -134,29 +144,43 @@ impl PushComponent {
         Ok(())
     }
 
-    fn get_progress(&self) -> (String, u8) {
-        self.progress.as_ref().map_or(
+    ///
+    pub const fn any_work_pending(&self) -> bool {
+        self.pending
+    }
+
+    ///
+    pub fn get_progress(
+        progress: &Option<RemoteProgress>,
+    ) -> (String, u8) {
+        progress.as_ref().map_or(
             (strings::PUSH_POPUP_PROGRESS_NONE.into(), 0),
             |progress| {
                 (
                     Self::progress_state_name(&progress.state),
-                    progress.progress,
+                    progress.get_progress_percent(),
                 )
             },
         )
     }
 
-    fn progress_state_name(state: &PushProgressState) -> String {
+    fn progress_state_name(state: &RemoteProgressState) -> String {
         match state {
-            PushProgressState::PackingAddingObject => {
+            RemoteProgressState::PackingAddingObject => {
                 strings::PUSH_POPUP_STATES_ADDING
             }
-            PushProgressState::PackingDeltafiction => {
+            RemoteProgressState::PackingDeltafiction => {
                 strings::PUSH_POPUP_STATES_DELTAS
             }
-            PushProgressState::Pushing => {
+            RemoteProgressState::Pushing => {
                 strings::PUSH_POPUP_STATES_PUSHING
             }
+            RemoteProgressState::Transfer => {
+                strings::PUSH_POPUP_STATES_TRANSFER
+            }
+            RemoteProgressState::Done => {
+                strings::PUSH_POPUP_STATES_DONE
+            }
         }
         .into()
     }
@@ -169,7 +193,8 @@ impl DrawableComponent for PushComponent {
         rect: Rect,
     ) -> Result<()> {
         if self.visible {
-            let (state, progress) = self.get_progress();
+            let (state, progress) =
+                Self::get_progress(&self.progress);
 
             let area = ui::centered_rect_absolute(30, 3, f.size());
 
@@ -180,7 +205,11 @@ impl DrawableComponent for PushComponent {
                     .block(
                         Block::default()
                             .title(Span::styled(
-                                strings::PUSH_POPUP_MSG,
+                                if self.force {
+                                    strings::FORCE_PUSH_POPUP_MSG
+                                } else {
+                                    strings::PUSH_POPUP_MSG
+                                },
                                 self.theme.title(true),
                             ))
                             .borders(Borders::ALL)
@@ -223,22 +252,22 @@ impl Component for PushComponent {
     fn event(&mut self, ev: Event) -> Result<bool> {
         if self.visible {
             if let Event::Key(e) = ev {
-                if e == self.key_config.exit_popup {
-                    self.hide();
-                }
-                if self.input_cred.event(ev)? {
-                    return Ok(true);
-                } else if e == self.key_config.enter {
-                    if self.input_cred.is_visible()
-                        && self.input_cred.get_cred().is_complete()
+                if self.input_cred.is_visible() {
+                    self.input_cred.event(ev)?;
+
+                    if self.input_cred.get_cred().is_complete()
+                        || !self.input_cred.is_visible()
                     {
-                        self.push_to_remote(Some(
-                            self.input_cred.get_cred().clone(),
-                        ))?;
+                        self.push_to_remote(
+                            Some(self.input_cred.get_cred().clone()),
+                            self.force,
+                        )?;
                         self.input_cred.hide();
-                    } else {
-                        self.hide();
                     }
+                } else if e == self.key_config.exit_popup
+                    && !self.pending
+                {
+                    self.hide();
                 }
             }
             return Ok(true);
