@@ -38,7 +38,7 @@ impl DrawableComponent for ResetComponent {
                 self.theme.text_danger(),
             );
 
-            let area = ui::centered_rect(30, 20, f.size());
+            let area = ui::centered_rect(50, 20, f.size());
             f.render_widget(Clear, area);
             f.render_widget(
                 popup_paragraph(&title, txt, &self.theme, true),
@@ -57,7 +57,7 @@ impl Component for ResetComponent {
         _force_all: bool,
     ) -> CommandBlocking {
         out.push(CommandInfo::new(
-            strings::commands::reset_confirm(&self.key_config),
+            strings::commands::confirm_action(&self.key_config),
             true,
             self.visible,
         ));
@@ -151,6 +151,10 @@ impl ResetComponent {
                     strings::confirm_title_reset(&self.key_config),
                     strings::confirm_msg_resethunk(&self.key_config),
                 ),
+                Action::ResetLines(_, lines) => (
+                    strings::confirm_title_reset(&self.key_config),
+                    strings::confirm_msg_reset_lines(&self.key_config,lines.len()),
+                ),
                 Action::DeleteBranch(branch_ref) => (
                     strings::confirm_title_delete_branch(
                         &self.key_config,
@@ -160,6 +164,19 @@ impl ResetComponent {
                         branch_ref,
                     ),
                 ),
+                Action::ForcePush(branch, _force) => (
+                    strings::confirm_title_force_push(
+                        &self.key_config,
+                    ),
+                    strings::confirm_msg_force_push(
+                        &self.key_config,
+                        branch.rsplit('/').next().expect("There was no / in the head reference which is impossible in git"),
+                    ),
+                ),
+                Action::PullMerge{incoming,rebase} => (
+                    strings::confirm_title_merge(&self.key_config,*rebase),
+                    strings::confirm_msg_merge(&self.key_config,*incoming,*rebase),
+                ),
             };
         }
 
