@@ -11,13 +11,13 @@ use crate::{
 use anyhow::Result;
 use crossterm::event::{Event, KeyCode, KeyModifiers};
 use itertools::Itertools;
-use std::{collections::HashMap, ops::Range};
+use std::{cell::Cell, collections::HashMap, ops::Range};
 use tui::{
     backend::Backend,
-    layout::Rect,
+    layout::{Alignment, Rect},
     style::Modifier,
     text::{Spans, Text},
-    widgets::Clear,
+    widgets::{Clear, Paragraph},
     Frame,
 };
 
@@ -34,10 +34,12 @@ pub struct TextInputComponent {
     default_msg: String,
     msg: String,
     visible: bool,
+    show_char_count: bool,
     theme: SharedTheme,
     key_config: SharedKeyConfig,
     cursor_position: usize,
     input_type: InputType,
+    current_area: Cell<Rect>,
 }
 
 impl TextInputComponent {
@@ -47,16 +49,19 @@ impl TextInputComponent {
         key_config: SharedKeyConfig,
         title: &str,
         default_msg: &str,
+        show_char_count: bool,
     ) -> Self {
         Self {
             msg: String::new(),
             visible: false,
             theme,
             key_config,
+            show_char_count,
             title: title.to_string(),
             default_msg: default_msg.to_string(),
             cursor_position: 0,
             input_type: InputType::Multiline,
+            current_area: Cell::new(Rect::default()),
         }
     }
 
@@ -79,6 +84,11 @@ impl TextInputComponent {
         &self.msg
     }
 
+    /// screen area (last time we got drawn)
+    pub fn get_area(&self) -> Rect {
+        self.current_area.get()
+    }
+
     /// Move the cursor right one char.
     fn incr_cursor(&mut self) {
         if let Some(pos) = self.next_char_position() {
@@ -209,6 +219,28 @@ impl TextInputComponent {
             _ => self.msg[range].to_owned(),
         }
     }
+
+    fn draw_char_count<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
+        let count = self.msg.len();
+        if count > 0 {
+            let w = Paragraph::new(format!("[{} chars]", count))
+                .alignment(Alignment::Right);
+
+            let mut rect = {
+                let mut rect = r;
+                rect.y += rect.height.saturating_sub(1);
+                rect
+            };
+
+            rect.x += 1;
+            rect.width = rect.width.saturating_sub(2);
+            rect.height = rect
+                .height
+                .saturating_sub(rect.height.saturating_sub(1));
+
+            f.render_widget(w, rect);
+        }
+    }
 }
 
 // merges last line of `txt` with first of `append` so we do not generate unneeded newlines
@@ -269,6 +301,12 @@ impl DrawableComponent for TextInputComponent {
                 ),
                 area,
             );
+
+            if self.show_char_count {
+                self.draw_char_count(f, area);
+            }
+
+            self.current_area.set(area);
         }
 
         Ok(())
@@ -369,6 +407,7 @@ mod tests {
             SharedKeyConfig::default(),
             "",
             "",
+            false,
         );
 
         comp.set_text(String::from("a\nb"));
@@ -389,6 +428,7 @@ mod tests {
             SharedKeyConfig::default(),
             "",
             "",
+            false,
         );
         let theme = SharedTheme::default();
         let underlined = theme
@@ -411,6 +451,7 @@ mod tests {
             SharedKeyConfig::default(),
             "",
             "",
+            false,
         );
         let theme = SharedTheme::default();
         let underlined_whitespace = theme
@@ -444,6 +485,7 @@ mod tests {
             SharedKeyConfig::default(),
             "",
             "",
+            false,
         );
 
         let theme = SharedTheme::default();
@@ -467,12 +509,13 @@ mod tests {
     }
 
     #[test]
-    fn test_invisable_newline() {
+    fn test_invisible_newline() {
         let mut comp = TextInputComponent::new(
             SharedTheme::default(),
             SharedKeyConfig::default(),
             "",
             "",
+            false,
         );
 
         let theme = SharedTheme::default();
