@@ -1,13 +1,16 @@
+//TODO: remove once fixed https://github.com/rust-lang/rust-clippy/issues/6818
+#![allow(clippy::use_self)]
+
 use crate::get_app_config_path;
 use anyhow::Result;
 use crossterm::event::{KeyCode, KeyEvent, KeyModifiers};
 use ron::{
-    de::from_bytes,
+    self,
     ser::{to_string_pretty, PrettyConfig},
 };
 use serde::{Deserialize, Serialize};
 use std::{
-    fs::File,
+    fs::{self, File},
     io::{Read, Write},
     path::PathBuf,
     rc::Rc,
@@ -23,8 +26,6 @@ pub struct KeyConfig {
     pub tab_stashes: KeyEvent,
     pub tab_toggle: KeyEvent,
     pub tab_toggle_reverse: KeyEvent,
-    pub focus_workdir: KeyEvent,
-    pub focus_stage: KeyEvent,
     pub focus_right: KeyEvent,
     pub focus_left: KeyEvent,
     pub focus_above: KeyEvent,
@@ -49,6 +50,8 @@ pub struct KeyConfig {
     pub status_stage_all: KeyEvent,
     pub status_reset_item: KeyEvent,
     pub status_ignore_file: KeyEvent,
+    pub diff_stage_lines: KeyEvent,
+    pub diff_reset_lines: KeyEvent,
     pub stashing_save: KeyEvent,
     pub stashing_toggle_untracked: KeyEvent,
     pub stashing_toggle_index: KeyEvent,
@@ -63,7 +66,8 @@ pub struct KeyConfig {
     pub select_branch: KeyEvent,
     pub delete_branch: KeyEvent,
     pub push: KeyEvent,
-    pub fetch: KeyEvent,
+    pub force_push: KeyEvent,
+    pub pull: KeyEvent,
 }
 
 #[rustfmt::skip]
@@ -76,8 +80,6 @@ impl Default for KeyConfig {
 			tab_stashes: KeyEvent { code: KeyCode::Char('4'), modifiers: KeyModifiers::empty()},
 			tab_toggle: KeyEvent { code: KeyCode::Tab, modifiers: KeyModifiers::empty()},
 			tab_toggle_reverse: KeyEvent { code: KeyCode::BackTab, modifiers: KeyModifiers::SHIFT},
-			focus_workdir: KeyEvent { code: KeyCode::Char('w'), modifiers: KeyModifiers::empty()},
-			focus_stage: KeyEvent { code: KeyCode::Char('s'), modifiers: KeyModifiers::empty()},
 			focus_right: KeyEvent { code: KeyCode::Right, modifiers: KeyModifiers::empty()},
 			focus_left: KeyEvent { code: KeyCode::Left, modifiers: KeyModifiers::empty()},
 			focus_above: KeyEvent { code: KeyCode::Up, modifiers: KeyModifiers::empty()},
@@ -101,7 +103,9 @@ impl Default for KeyConfig {
 			edit_file: KeyEvent { code: KeyCode::Char('e'), modifiers: KeyModifiers::empty()},
 			status_stage_all: KeyEvent { code: KeyCode::Char('a'), modifiers: KeyModifiers::empty()},
 			status_reset_item: KeyEvent { code: KeyCode::Char('D'), modifiers: KeyModifiers::SHIFT},
+            diff_reset_lines: KeyEvent { code: KeyCode::Char('d'), modifiers: KeyModifiers::empty()},
 			status_ignore_file: KeyEvent { code: KeyCode::Char('i'), modifiers: KeyModifiers::empty()},
+            diff_stage_lines: KeyEvent { code: KeyCode::Char('s'), modifiers: KeyModifiers::empty()},
 			stashing_save: KeyEvent { code: KeyCode::Char('s'), modifiers: KeyModifiers::empty()},
 			stashing_toggle_untracked: KeyEvent { code: KeyCode::Char('u'), modifiers: KeyModifiers::empty()},
 			stashing_toggle_index: KeyEvent { code: KeyCode::Char('i'), modifiers: KeyModifiers::empty()},
@@ -116,20 +120,21 @@ impl Default for KeyConfig {
             select_branch: KeyEvent { code: KeyCode::Char('b'), modifiers: KeyModifiers::NONE},
             delete_branch: KeyEvent{code: KeyCode::Char('D'), modifiers: KeyModifiers::SHIFT},
             push: KeyEvent { code: KeyCode::Char('p'), modifiers: KeyModifiers::empty()},
-            fetch: KeyEvent { code: KeyCode::Char('f'), modifiers: KeyModifiers::empty()},
+            force_push: KeyEvent { code: KeyCode::Char('P'), modifiers: KeyModifiers::SHIFT},
+            pull: KeyEvent { code: KeyCode::Char('f'), modifiers: KeyModifiers::empty()},
         }
     }
 }
+
 impl KeyConfig {
-    fn save(&self) -> Result<()> {
-        let config_file = Self::get_config_file()?;
-        let mut file = File::create(config_file)?;
+    fn save(&self, file: PathBuf) -> Result<()> {
+        let mut file = File::create(file)?;
         let data = to_string_pretty(self, PrettyConfig::default())?;
         file.write_all(data.as_bytes())?;
         Ok(())
     }
 
-    fn get_config_file() -> Result<PathBuf> {
+    pub fn get_config_file() -> Result<PathBuf> {
         let app_home = get_app_config_path()?;
         Ok(app_home.join("key_config.ron"))
     }
@@ -138,115 +143,122 @@ impl KeyConfig {
         let mut f = File::open(config_file)?;
         let mut buffer = Vec::new();
         f.read_to_end(&mut buffer)?;
-        Ok(from_bytes(&buffer)?)
+        Ok(ron::de::from_bytes(&buffer)?)
     }
 
-    fn init_internal() -> Result<Self> {
-        let file = Self::get_config_file()?;
+    pub fn init(file: PathBuf) -> Result<Self> {
         if file.exists() {
-            Ok(Self::read_file(file)?)
+            match Self::read_file(file.clone()) {
+                Err(e) => {
+                    let config_path = file.clone();
+                    let config_path_old =
+                        format!("{}.old", file.to_string_lossy());
+                    fs::rename(
+                        config_path.clone(),
+                        config_path_old.clone(),
+                    )?;
+
+                    Self::default().save(file)?;
+
+                    Err(anyhow::anyhow!("{}\n Old file was renamed to {:?}.\n Defaults loaded and saved as {:?}",
+                        e,config_path_old,config_path.to_string_lossy()))
+                }
+                Ok(res) => Ok(res),
+            }
         } else {
-            let def = Self::default();
-            if def.save().is_err() {
-                log::warn!(
-                    "failed to store default key config to disk."
+            Self::default().save(file)?;
+            Ok(Self::default())
+        }
+    }
+
+    //TODO: make this configurable (https://github.com/extrawurst/gitui/issues/465)
+    #[allow(clippy::unused_self)]
+    const fn get_key_symbol(&self, k: KeyCode) -> &str {
+        match k {
+            KeyCode::Enter => "\u{23ce}",     //⏎
+            KeyCode::Left => "\u{2190}",      //←
+            KeyCode::Right => "\u{2192}",     //→
+            KeyCode::Up => "\u{2191}",        //↑
+            KeyCode::Down => "\u{2193}",      //↓
+            KeyCode::Backspace => "\u{232b}", //⌫
+            KeyCode::Home => "\u{2912}",      //⤒
+            KeyCode::End => "\u{2913}",       //⤓
+            KeyCode::PageUp => "\u{21de}",    //⇞
+            KeyCode::PageDown => "\u{21df}",  //⇟
+            KeyCode::Tab => "\u{21e5}",       //⇥
+            KeyCode::BackTab => "\u{21e4}",   //⇤
+            KeyCode::Delete => "\u{2326}",    //⌦
+            KeyCode::Insert => "\u{2380}",    //⎀
+            KeyCode::Esc => "\u{238b}",       //⎋
+            _ => "?",
+        }
+    }
+
+    pub fn get_hint(&self, ev: KeyEvent) -> String {
+        match ev.code {
+            KeyCode::Down
+            | KeyCode::Up
+            | KeyCode::Right
+            | KeyCode::Left
+            | KeyCode::Enter
+            | KeyCode::Backspace
+            | KeyCode::Home
+            | KeyCode::End
+            | KeyCode::PageUp
+            | KeyCode::PageDown
+            | KeyCode::Tab
+            | KeyCode::BackTab
+            | KeyCode::Delete
+            | KeyCode::Insert
+            | KeyCode::Esc => {
+                format!(
+                    "{}{}",
+                    Self::get_modifier_hint(ev.modifiers),
+                    self.get_key_symbol(ev.code)
                 )
             }
-            Ok(def)
-        }
-    }
-
-    pub fn init() -> Self {
-        match Self::init_internal() {
-            Ok(v) => v,
-            Err(e) => {
-                log::error!("failed loading key binding: {}", e);
-                Self::default()
+            KeyCode::Char(c) => {
+                format!(
+                    "{}{}",
+                    Self::get_modifier_hint(ev.modifiers),
+                    c
+                )
             }
+            KeyCode::F(u) => {
+                format!(
+                    "{}F{}",
+                    Self::get_modifier_hint(ev.modifiers),
+                    u
+                )
+            }
+            KeyCode::Null => Self::get_modifier_hint(ev.modifiers),
         }
     }
-}
 
-// The hint follows apple design
-// http://xahlee.info/comp/unicode_computing_symbols.html
-pub fn get_hint(ev: KeyEvent) -> String {
-    match ev.code {
-        KeyCode::Char(c) => {
-            format!("{}{}", get_modifier_hint(ev.modifiers), c)
+    //TODO: make customizable (see https://github.com/extrawurst/gitui/issues/465)
+    fn get_modifier_hint(modifier: KeyModifiers) -> String {
+        match modifier {
+            KeyModifiers::CONTROL => "^".to_string(),
+            KeyModifiers::SHIFT => {
+                "\u{21e7}".to_string() //⇧
+            }
+            KeyModifiers::ALT => {
+                "\u{2325}".to_string() //⌥
+            }
+            _ => String::new(),
         }
-        KeyCode::Enter => {
-            format!("{}\u{23ce}", get_modifier_hint(ev.modifiers)) //⏎
-        }
-        KeyCode::Left => {
-            format!("{}\u{2190}", get_modifier_hint(ev.modifiers)) //←
-        }
-        KeyCode::Right => {
-            format!("{}\u{2192}", get_modifier_hint(ev.modifiers)) //→
-        }
-        KeyCode::Up => {
-            format!("{}\u{2191}", get_modifier_hint(ev.modifiers)) //↑
-        }
-        KeyCode::Down => {
-            format!("{}\u{2193}", get_modifier_hint(ev.modifiers)) //↓
-        }
-        KeyCode::Backspace => {
-            format!("{}\u{232b}", get_modifier_hint(ev.modifiers)) //⌫
-        }
-        KeyCode::Home => {
-            format!("{}\u{2912}", get_modifier_hint(ev.modifiers)) //⤒
-        }
-        KeyCode::End => {
-            format!("{}\u{2913}", get_modifier_hint(ev.modifiers)) //⤓
-        }
-        KeyCode::PageUp => {
-            format!("{}\u{21de}", get_modifier_hint(ev.modifiers)) //⇞
-        }
-        KeyCode::PageDown => {
-            format!("{}\u{21df}", get_modifier_hint(ev.modifiers)) //⇟
-        }
-        KeyCode::Tab => {
-            format!("{}\u{21e5}", get_modifier_hint(ev.modifiers)) //⇥
-        }
-        KeyCode::BackTab => {
-            format!("{}\u{21e4}", get_modifier_hint(ev.modifiers)) //⇤
-        }
-        KeyCode::Delete => {
-            format!("{}\u{2326}", get_modifier_hint(ev.modifiers)) //⌦
-        }
-        KeyCode::Insert => {
-            format!("{}\u{2380}", get_modifier_hint(ev.modifiers)) //⎀
-        }
-        KeyCode::Esc => {
-            format!("{}\u{238b}", get_modifier_hint(ev.modifiers)) //⎋
-        }
-        KeyCode::F(u) => {
-            format!("{}F{}", get_modifier_hint(ev.modifiers), u)
-        }
-        KeyCode::Null => get_modifier_hint(ev.modifiers),
-    }
-}
-
-fn get_modifier_hint(modifier: KeyModifiers) -> String {
-    match modifier {
-        KeyModifiers::CONTROL => "^".to_string(),
-        KeyModifiers::SHIFT => {
-            "\u{21e7}".to_string() //⇧
-        }
-        KeyModifiers::ALT => {
-            "\u{2325}".to_string() //⌥
-        }
-        _ => String::new(),
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::{get_hint, KeyConfig};
+    use super::KeyConfig;
     use crossterm::event::{KeyCode, KeyEvent, KeyModifiers};
 
     #[test]
     fn test_get_hint() {
-        let h = get_hint(KeyEvent {
+        let config = KeyConfig::default();
+        let h = config.get_hint(KeyEvent {
             code: KeyCode::Char('c'),
             modifiers: KeyModifiers::CONTROL,
         });
