@@ -9,7 +9,7 @@ use ron::{
 };
 use serde::{Deserialize, Serialize};
 use std::{
-    fs::File,
+    fs::{self, File},
     io::{Read, Write},
     path::PathBuf,
     rc::Rc,
@@ -135,6 +135,9 @@ impl Theme {
             StatusItemType::Renamed => {
                 Style::default().fg(self.diff_file_moved)
             }
+            StatusItemType::Conflicted => Style::default()
+                .fg(self.diff_file_modified)
+                .add_modifier(Modifier::BOLD),
             StatusItemType::Typechange => Style::default(),
         };
 
@@ -247,21 +250,29 @@ impl Theme {
         Ok(from_bytes(&buffer)?)
     }
 
-    fn init_internal(theme: PathBuf) -> Result<Self> {
-        if theme.exists() {
-            Ok(Self::read_file(theme)?)
-        } else {
-            // This will only be called when theme.ron doesn't already exists
-            let def = Self::default();
-            if def.save(theme).is_err() {
-                log::warn!("failed to store default theme to disk.")
-            }
-            Ok(def)
-        }
-    }
+    pub fn init(file: PathBuf) -> Result<Self> {
+        if file.exists() {
+            match Self::read_file(file.clone()) {
+                Err(e) => {
+                    let config_path = file.clone();
+                    let config_path_old =
+                        format!("{}.old", file.to_string_lossy());
+                    fs::rename(
+                        config_path.clone(),
+                        config_path_old.clone(),
+                    )?;
 
-    pub fn init(theme_path: PathBuf) -> Self {
-        Self::init_internal(theme_path).unwrap_or_default()
+                    Self::default().save(file)?;
+
+                    Err(anyhow::anyhow!("{}\n Old file was renamed to {:?}.\n Defaults loaded and saved as {:?}",
+                        e,config_path_old,config_path.to_string_lossy()))
+                }
+                Ok(res) => Ok(res),
+            }
+        } else {
+            Self::default().save(file)?;
+            Ok(Self::default())
+        }
     }
 }
 
